@@ -7,17 +7,21 @@
 #include <stdint.h>
 #include <stdio.h>
 
+#ifndef RAFT_API
 #define RAFT_API __attribute__((visibility("default")))
+#endif
 
 /**
  * Version.
  */
-#define RAFT_VERSION_MAJOR    1
-#define RAFT_VERSION_MINOR    17
-#define RAFT_VERSION_RELEASE  1
-#define RAFT_VERSION_NUMBER (RAFT_VERSION_MAJOR *100*100 + RAFT_VERSION_MINOR *100 + RAFT_VERSION_RELEASE)
+#define RAFT_VERSION_MAJOR 0
+#define RAFT_VERSION_MINOR 18
+#define RAFT_VERSION_RELEASE 0
+#define RAFT_VERSION_NUMBER                                      \
+    (RAFT_VERSION_MAJOR * 100 * 100 + RAFT_VERSION_MINOR * 100 + \
+     RAFT_VERSION_RELEASE)
 
-int raft_version_number (void);
+int raft_version_number(void);
 
 /**
  * Error codes.
@@ -75,6 +79,11 @@ typedef unsigned long long raft_index;
  */
 typedef unsigned long long raft_time;
 
+/**
+ * Hold the features a raft node is capable of.
+ */
+typedef uint64_t raft_flags;
+
 /**
  * A data buffer.
  */
@@ -84,7 +93,6 @@ struct raft_buffer
     size_t len; /* Length of the buffer. */
 };
 
-
 /**
  * Server role codes.
  */
@@ -228,9 +236,9 @@ struct raft_request_vote
 struct raft_request_vote_result
 {
     int version;
-    raft_term term;        /* Receiver's current term (candidate updates itself). */
-    bool vote_granted;     /* True means candidate received vote. */
-    bool pre_vote;         /* The response to a pre-vote RequestVote or not. */
+    raft_term term;    /* Receiver's current term (candidate updates itself). */
+    bool vote_granted; /* True means candidate received vote. */
+    bool pre_vote;     /* The response to a pre-vote RequestVote or not. */
 };
 #define RAFT_REQUEST_VOTE_RESULT_VERSION 2
 
@@ -261,8 +269,9 @@ struct raft_append_entries_result
     raft_term term;            /* Receiver's current_term. */
     raft_index rejected;       /* If non-zero, the index that was rejected. */
     raft_index last_log_index; /* Receiver's last log entry index, as hint. */
+    raft_flags features;       /* Feature flags. */
 };
-#define RAFT_APPEND_ENTRIES_RESULT_VERSION 0
+#define RAFT_APPEND_ENTRIES_RESULT_VERSION 1
 
 /**
  * Hold the arguments of an InstallSnapshot RPC (figure 5.3).
@@ -352,8 +361,8 @@ struct raft_message
  * Hold the details of a snapshot.
  * The user-provided raft_buffer structs should provide the user with enough
  * flexibility to adapt/evolve snapshot formats.
- * If this struct would NEED to be adapted in the future, raft can always move to
- * a new struct with a new name and a new raft_io version.
+ * If this struct would NEED to be adapted in the future, raft can always move
+ * to a new struct with a new name and a new raft_io version.
  */
 struct raft_snapshot
 {
@@ -545,10 +554,10 @@ struct raft_io
  * version 3:
  * Adds support for async snapshots through the `snapshot_async` function.
  * When this method is provided, raft will call `snapshot` in the main loop,
- * and when successful, will call `snapshot_async` by means of the `io->async_work` method.
- * The callback to `io->async_work` will in turn call `snapshot_finalize`
- * in the main loop when the work has completed independent of the return value
- * of `snapshot_async`.
+ * and when successful, will call `snapshot_async` using the `io->async_work`
+ * method, so blocking I/O calls are allowed in the implementation. After the
+ * `snapshot_async` completes, `snapshot_finalize` will be called in the main
+ * loop, independent of the return value of `snapshot_async`.
  * An implementation that does not use asynchronous snapshots MUST set
  * `snapshot_async` to NULL.
  * All memory allocated by the snapshot routines MUST be freed by the snapshot
@@ -629,30 +638,51 @@ struct raft
      * If a server is voting, the log entry with index 1 must always contain the
      * first committed configuration.
      *
+     * At all times #configuration_committed_index is either zero or is the
+     * index of the most recent log entry of type #RAFT_CHANGE that we know to
+     * be committed. That means #configuration_committed_index is always equal
+     * or lower than #commit_index.
+     *
+     * At all times #configuration_uncommitted_index is either zero or is the
+     * index of an uncommitted log entry of type #RAFT_CHANGE. There can be at
+     * most one uncommitted entry of type #RAFT_CHANGE because we allow only one
+     * configuration change at a time.
+     *
+     * At all times #configuration_last_snapshot is a copy of the configuration
+     * contained the most recent snapshot, if any.
+     *
      * The possible scenarios are:
      *
-     * 1. #configuration_index and #configuration_uncommitted_index are both
-     *    zero. This should only happen when a brand new server starts joining a
-     *    cluster and is waiting to receive log entries from the current
-     *    leader. In this case #configuration and #configuration_previous
-     *    must be empty and have no servers.
+     * 1. #configuration_committed_index and #configuration_uncommitted_index
+     *    are both zero. This should only happen when a brand new server starts
+     *    joining a cluster and is waiting to receive log entries from the
+     *    current leader. In this case #configuration and
+     *    #configuration_last_snapshot must be empty and have no servers.
      *
-     * 2. #configuration_index is non-zero while #configuration_uncommitted_index
-     *    is zero. In this case the content of #configuration must match the one
-     *    of the log entry at #configuration_index.
+     * 2. #configuration_committed_index is non-zero and
+     *    #configuration_uncommitted_index is zero. This means that
+     *    #configuration is committed and there is no pending configuration
+     *    change. The content of #configuration must match the one of the log
+     *    entry at #configuration_committed_index.
      *
-     * 3. #configuration_index and #configuration_uncommitted_index are both
-     *    non-zero, with the latter being greater than the former. In this case
-     *    the content of #configuration must match the one of the log entry at
-     *    #configuration_uncommitted_index.
+     * 3. #configuration_committed_index and #configuration_uncommitted_index
+     *    are both non-zero, with the latter being greater than the former. This
+     *    means that #configuration is uncommitted and represents a pending
+     *    configuration change. The content of #configuration must match the one
+     *    of the log entry at #configuration_uncommitted_index.
      *
-     * 4. In case the previous - committed - configuration can no longer be found
-     *    in the log e.g. after truncating the log when taking or installing a
-     *    snapshot, `configuration_previous` will contain a copy of it.
+     * When a snapshot is taken, a copy of the most recent configuration known
+     * to be committed (i.e. the configuration contained in the log entry at
+     * #configuration_committed_index) is saved in #configuration_last_snapshot,
+     * so it can be easily retrieved in case the log gets truncated because of
+     * compaction and does not contain the entry at
+     * #configuration_committed_index anymore. Likewise, if a snapshot is
+     * restored its associated configuration is saved in
+     * #configuration_last_snapshot.
      */
     struct raft_configuration configuration;
-    struct raft_configuration configuration_previous;
-    raft_index configuration_index;
+    struct raft_configuration configuration_last_snapshot;
+    raft_index configuration_committed_index;
     raft_index configuration_uncommitted_index;
 
     /*
@@ -690,8 +720,8 @@ struct raft
     unsigned heartbeat_timeout;
 
     /*
-     * When the leader sends an InstallSnapshot RPC to a follower it will consider
-     * the RPC as failed after this timeout and retry.
+     * When the leader sends an InstallSnapshot RPC to a follower it will
+     * consider the RPC as failed after this timeout and retry.
      */
     unsigned install_snapshot_timeout;
 
@@ -719,7 +749,8 @@ struct raft
                 raft_id id;
                 char *address;
             } current_leader;
-            uint64_t reserved[8];                 /* Future use */
+            uint64_t append_in_flight_count;
+            uint64_t reserved[7]; /* Future use */
         } follower_state;
         struct
         {
@@ -900,10 +931,16 @@ RAFT_API void raft_set_max_catch_up_round_duration(struct raft *r,
 RAFT_API const char *raft_errmsg(struct raft *r);
 
 /**
- * Return the code of the current raft state.
+ * Return the code of the current raft state (follower/candidate/leader).
  */
 RAFT_API int raft_state(struct raft *r);
 
+/**
+ * Return the code of the current raft role (spare/standby/voter),
+ * or -1 if this server is not in the current configuration.
+ */
+RAFT_API int raft_role(struct raft *r);
+
 /**
  * Return the ID and address of the current known leader, if any.
  */
@@ -933,7 +970,7 @@ RAFT_API raft_index raft_last_applied(struct raft *r);
     uint8_t req_id[16];    \
     uint8_t client_id[16]; \
     uint8_t unique_id[16]; \
-    uint64_t reserved[4]   \
+    uint64_t reserved[4]
 
 /**
  * Asynchronous request to append a new command entry to the log and apply it to
